@@ -10,7 +10,7 @@ test_that("PlotCatalogToPdf.SBS96Catalog function", {
   out <-
     PlotCatalogToPdf(catalog.counts, 
                      file = file.path(tempdir(), "PlotCatSBS96.counts.test.pdf"))
-  expect_equal(out, TRUE)
+  expect_equal(out$plot.success, TRUE)
 
   catalog.density <-
     TransformCatalog(catalog.counts, target.ref.genome = "GRCh37",
@@ -18,7 +18,7 @@ test_that("PlotCatalogToPdf.SBS96Catalog function", {
                      target.catalog.type = "density")
   out <-
     PlotCatalogToPdf(catalog.density, file = file.path(tempdir(), "PlotCatSBS96.density.test.pdf"))
-  expect_equal(out, TRUE)
+  expect_equal(out$plot.success, TRUE)
 
   catalog.counts.signature <-
     TransformCatalog(catalog.counts, target.ref.genome = "GRCh37",
@@ -27,7 +27,7 @@ test_that("PlotCatalogToPdf.SBS96Catalog function", {
   out <-
     PlotCatalogToPdf(catalog.counts.signature,
                      file = file.path(tempdir(), "PlotCatSBS96.counts.signature.test.pdf"))
-  expect_equal(out, TRUE)
+  expect_equal(out$plot.success, TRUE)
 
   catalog.density.signature <-
     TransformCatalog(catalog.counts, target.ref.genome = "GRCh37",
@@ -36,11 +36,30 @@ test_that("PlotCatalogToPdf.SBS96Catalog function", {
   out <-
     PlotCatalogToPdf(catalog.density.signature,
                      file = file.path(tempdir(), "PlotCatSBS96.density.signature.test.pdf"))
-  expect_equal(out, TRUE)
+  expect_equal(out$plot.success, TRUE)
 
-  unlink(file.path(tempdir(), "PlotCatSBS96.counts.test.pdf"))
-  unlink(file.path(tempdir(), "PlotCatSBS96.density.test.pdf"))
-  unlink(file.path(tempdir(), "PlotCatSBS96.counts.signature.test.pdf"))
-  unlink(file.path(tempdir(), "PlotCatSBS96.density.signature.test.pdf"))
+  if (Sys.getenv("ICAMS.SAVE.TEST.PDF") != "") {
+    file.rename(from = file.path(tempdir(), 
+                                 "PlotCatSBS96.counts.test.pdf"),
+                to   = file.path("pdfs-for-comparision-SBS96",
+                                 "PlotCatSBS96.counts.test.pdf"))
+    file.rename(from = file.path(tempdir(), 
+                                 "PlotCatSBS96.density.test.pdf"),
+                to   = file.path("pdfs-for-comparision-SBS96",
+                                 "PlotCatSBS96.density.test.pdf"))
+    file.rename(from = file.path(tempdir(), 
+                                 "PlotCatSBS96.counts.signature.test.pdf"),
+                to   = file.path("pdfs-for-comparision-SBS96",
+                                 "PlotCatSBS96.counts.signature.test.pdf"))
+    file.rename(from = file.path(tempdir(), 
+                                 "PlotCatSBS96.density.signature.test.pdf"),
+                to   = file.path("pdfs-for-comparision-SBS96",
+                                 "PlotCatSBS96.density.signature.test.pdf"))
+  } else {
+    unlink(file.path(tempdir(), "PlotCatSBS96.counts.test.pdf"))
+    unlink(file.path(tempdir(), "PlotCatSBS96.density.test.pdf"))
+    unlink(file.path(tempdir(), "PlotCatSBS96.counts.signature.test.pdf"))
+    unlink(file.path(tempdir(), "PlotCatSBS96.density.signature.test.pdf"))
+  }
   graphics.off()
 })
